; ========================================
; |)emOS: A 512-byte OS with a dumb little
; random number generator and i186 compat-
; iblity, like  everything  else  I  code.
;
; Four commands: 'e','h','d','?'
; Assemble with nasm -o boot.bin demos.asm
; and dd or partcopy to a floppy sector 0!
; ========================================

	CPU 186

	mov ax,0x07C0		; set up segments
	mov ds,ax
	mov es,ax

	mov si,welcome
	call print_string

loop:
	mov si,prompt
	call print_string

	mov di,buffer
	call get_string

	mov si,buffer
	cmp byte [si],0		; blank line?
	je short loop		; yes, ignore it

	mov si,buffer
	mov di,cmd_hi	; "h" command
	cmpsb
	je short .helloworld

	mov si,buffer
	mov di,cmd_help		; "?" command
	cmpsb
	je short .help

	mov si,buffer
	mov di,cmd_dump		; "d" command
	cmpsb
	je short .dump

	mov si,buffer
	mov di,cmd_echospace	; "e " command
	mov cx,2
	rep cmpsb
	je .echoops

	mov si,buffer
	mov di,cmd_echo		; "e" command
	cmpsb
	je .echonoops

.error:
	mov si,badcommand
	call print_string 

	mov si,buffer
	call print_string

	call print_newline

	jmp short loop  

.helloworld:
	xor ah,ah
	int 1Ah		; Get number of ticks since midnight as a random seed

	shr cx,3	; Shirftcx right three places
	sub dx,cx	; Subtract new cx from dx
	
	mov word [temp2],dx
	xor ah,ah
	int 1Ah					; Get number of ticks since midnight again
	mov word [temp],dx		; Store lower word in .temp
	mov dx, word [temp2]

	mov ax,[temp]	; Store the word pointed to by .temp in ax
	ror al,3		; Roll al right three places
	add dx,ax		; Add ax to dx

	rol dx,2	; Roll dx left two places
	xchg dl,dh	; Flip dl and dh
	ror dl,3	; Roll dl right three places

	mov ax,dx
	call print_hex_word

	mov si,msg_random
	call print_string

	jmp loop

.help:
	mov si,msg_help
	call print_string

	jmp loop

.dump:
	mov si,reg_ax
	call print_string
	call print_hex_word

	mov si,reg_bx
	call print_string
	mov ax,bx
	call print_hex_word

	mov si,reg_cx
	call print_string
	mov ax,cx
	call print_hex_word

	mov si,reg_dx
	call print_string
	mov ax,dx
	call print_hex_word

	mov si,reg_cs
	call print_string
	push cs
	pop ax
	call print_hex_word

	mov si,reg_ds
	call print_string
	mov ax,ds
	call print_hex_word
	
	call print_newline

	jmp loop

.echonoops:
	mov si,buffer
	add si,1
	cmp byte [si],0
	jne .error

	jmp loop

.echoops:
	mov si,buffer
	add si,2
	call print_string

	call print_newline

	jmp loop

; ========================================

crlf db 0x0D,0x0A,0
welcome db '|)emOS',0x0D,0x0A,0
msg_random db 'h',0x0D,0x0A,0
badcommand db 'bad cmd: ',0
prompt db '|)',0
cmd_hi db 'h',0
cmd_help db '?',0
cmd_echo db 'e',0
cmd_echospace db 'e ',0
cmd_dump db 'd',0
msg_help db 'cmds: h,d,e',0x0D,0x0A,0

reg_cs db ' CS=0x',0
reg_ds db ' DS/ES=0x',0
reg_ax db ' AX=0x',0
reg_bx db ' BX=0x',0
reg_cx db ' CX=0x',0
reg_dx db ' DX=0x',0

section .bss
temp resw 1
temp2 resw 1
buffer resb 64
section .text

; ========================================

print_string:
.loop:
	lodsb			; grab a byte from SI

	cmp al,0	; is al 0?
	je short .done	; yes, get out

	mov ah,0x0E
	int 0x10		; otherwise, print out the character!

	jmp short .loop

.done:
	ret

; ========================================

print_newline:
	mov si,crlf
	call print_string
	ret

; ========================================

get_string:
	xor cl,cl

.loop:
	xor ah,ah
	int 0x16	; wait for keypress

	cmp al,0x08		; backspace pressed?
	je short .backspace	; yes, handle it

	cmp al,0x0D		; enter pressed?
	je short .done		; yes, we're done

	cmp cl,63		; 63 chars inputted?
	je short .loop		; yes, only let in backspace and enter

	mov ah,0x0E
	int 0x10		; print out character

	stosb			; put character in buffer
	inc cl
	jmp short .loop

.backspace:
	cmp cl,0	; beginning of string?
	je short .loop	; yes, ignore the key

	dec di
	mov byte [di],0		; delete character
	dec cl				; decrement counter as well

	mov ah,0x0E
	mov al,0x08
	int 10h			; backspace on the screen

	mov al,' '
	int 10h		; blank character out

	mov al,0x08
	int 10h			; backspace again

	jmp short .loop		; go to the main loop

.done:
	mov al,0	; null terminator
	stosb

	mov ah,0x0E
	mov al,0x0D
	int 0x10
	mov al,0x0A
	int 0x10		; newline

	ret

; ========================================

print_hex_byte:
	mov [.temp],al
	shr al,4
	cmp al,10
	sbb al,69h
	das

	mov ah,0Eh
	int 10h

	mov al,[.temp]
	ror al,4
	shr al,4
	cmp al,10
	sbb al,69h
	das

	mov ah,0Eh
	int 10h

	ret

section .bss
.temp resb 1
section .text

; ========================================

print_hex_word:
	mov [.temp],al
	mov al,ah
	call print_hex_byte

	mov al,[.temp]
	call print_hex_byte
	
	ret

section .bss
.temp resb 1
section .text

times 510-($-$$) db 0	; really only need one byte of padding :D
	db 0x55
	db 0xAA		; signature